/*
 WSysTray was contributed to Absolute Power++:
   http://www.orbit.org/power/
 by Ivan Bowman. However, neither he, nor Powersoft take any
 responsibility whatsoever for this code or what you do with
 it - use it at your own risk. Similarly, neither Ivan Bowman
 nor Powersoft support this code, and will therefore not answer
 any questions regarding it.
*/

//
// WSysTray.cpp
//

#include "wpch.hpp"
#include "WSysTray.hpp"

WDefineSubclass( WSysTray, WWindow );

const int WSysTray_Message = WM_USER + 11;

static void PackNotifyIcon( WWindow * win, NOTIFYICONDATA & add, 
                            const WIcon * icon, const WChar * tip );

WSysTray::WSysTray()
/******************/
{
   WEventHandlerList  Events[] = {
      { WLeftButtonDoubleEvent,   (WEventHandler)&WSysTray::leftButtonDoubleEventHandler },
      { WDestroyEvent,            (WEventHandler)&WSysTray::destroyEventHandler },
      { WNullEvent,               NULL }
   };
   
   SetEventHandlerList( Events, this );
}

WSysTray::~WSysTray()
/*******************/
{
   Destroy();
}

WBool WSysTray::Create( const WIcon & icon, const WChar * tip )
/*************************************************************/
{
   WTraceMember();
   NOTIFYICONDATA  add;
   WBool           succ;
   
   succ = WWindow::Create( NULL, WRect( 0, 0, 0, 0 ),
      WTEXT("WClassTrayWindow"), NULLWSTYLE, NULLWSTYLE );
   WASSERT( succ );      
   
   PackNotifyIcon( this, add, &icon, tip );
   
   succ = Shell_NotifyIcon( NIM_ADD, &add );
   WASSERT( succ );
   return( TRUE );
}

virtual WBool WSysTray::ProcessMessage( 
   const WMessage &   inmsg, 
   WLong &            returns )
/**************************************/
{
   WMessage   msg( inmsg );
   if( msg.msg == WSysTray_Message ) {
      msg.msg = msg.lParam;
      msg.lParam = MAKELONG( msg.pt.x, msg.pt.y );
   } else if( msg.msg == WM_DESTROY ) {
      SetPopup( NULL );
   }
   
   if( GetHandle() != NULLHWND ) {
      return WWindow::ProcessMessage( msg, returns );
   } else {
      return FALSE;
   }
}

WBool WSysTray::SetIcon( const WIcon & icon )
/*******************************************/
{
   NOTIFYICONDATA  add;
   PackNotifyIcon( this, add, &icon, NULL );
   return Shell_NotifyIcon( NIM_MODIFY, &add );
}

WBool WSysTray::SetTip( const WChar * tip )
/*****************************************/
{
   NOTIFYICONDATA  add;
   PackNotifyIcon( this, add, NULL, tip );
   return Shell_NotifyIcon( NIM_MODIFY, &add );
}

WBool WSysTray::leftButtonDoubleEventHandler( WObject *, WEventData * )
/*********************************************************************/
{
   WPopupMenu *   popup;
   WInt           idx;
   WMenuItem *    item = NULL;
   popup = GetPopup();
   if( popup != NULL ) {
      idx = ::GetMenuDefaultItem( (HMENU)popup->GetHandle(), TRUE,
         GMDI_GOINTOPOPUPS | GMDI_USEDISABLED );
      item = popup->GetItem( idx );
      if( item != NULL ) {
         item->CallEventHandler( WClickEvent, item );
      }
   }
   return FALSE;
}

WBool WSysTray::destroyEventHandler( WObject *, WEventData * )
/************************************************************/
{
   NOTIFYICONDATA  add;
   PackNotifyIcon( this, add, NULL, NULL );
   Shell_NotifyIcon( NIM_DELETE, &add );
   return( FALSE );
}

static void PackNotifyIcon(
   WWindow *          win,
   NOTIFYICONDATA &   add, 
   const WIcon *      icon, 
   const WChar *      tip )
/**************************/
{
   add.cbSize             = sizeof( add );
   add.hWnd               = (HWND)win->GetHandle();
   add.uID                = 0;
   // Thanks to Chris Johnson <cjohnson@rocketship.com>
   //   for the correction to this line:
   add.uFlags             = NIF_MESSAGE;
   add.uCallbackMessage   = WSysTray_Message;

   if( icon != NULL ) {
      add.hIcon           = (HICON)icon->GetHandle();
   } else {
      add.hIcon           = NULL;
   }
   add.szTip[ 0 ]         = 0;
   
   if( tip != NULL ) {
      ::strncpy( add.szTip, tip, sizeof( add.szTip ) );
      add.szTip[ sizeof( add.szTip ) - 1 ] = 0;
   }
   
   if( icon != NULL ) {
      add.uFlags |= NIF_ICON;
   }
   if( tip != NULL ) {
      add.uFlags |= NIF_TIP;
   }
}